package cn.iocoder.yudao.module.bpm.controller.admin.oa.conversion;

import cn.iocoder.yudao.framework.apilog.core.annotation.ApiAccessLog;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.excel.core.util.ExcelUtils;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.conversion.vo.OaConversionPageReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.conversion.vo.OaConversionRespVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.conversion.vo.OaConversionSaveReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.task.vo.task.BpmTaskApproveReqVO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.conversion.OaConversionDO;
import cn.iocoder.yudao.module.bpm.service.oa.conversion.OaConversionService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.List;

import static cn.iocoder.yudao.framework.apilog.core.enums.OperateTypeEnum.EXPORT;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils.getLoginUserId;

@Tag(name = "管理后台 - 转正流程信息")
@RestController
@RequestMapping("/bpm/oa-conversion")
@Validated
public class OaConversionController {

    @Resource
    private OaConversionService oaConversionService;

    @PostMapping("/start")
    @Operation(summary = "提交或暂存转正流程信息")
    @PreAuthorize("@ss.hasPermission('bpm:oa-conversion:start')")
    public CommonResult<Long> startOaConversion(@Valid @RequestBody OaConversionSaveReqVO createReqVO) {
        return success(oaConversionService.startOaConversion(getLoginUserId(), createReqVO));
    }

    @PostMapping("/completeTure")
    @Operation(summary = "审批同意转正流程信息")
    @PreAuthorize("@ss.hasPermission('bpm:oa-entry:complete-true')")
    public CommonResult<Long> completeTrueOaConversion(@Valid @RequestBody BpmTaskApproveReqVO reqVO) {
        return success(oaConversionService.completeTrueOaConversion(getLoginUserId(), reqVO));
    }

    @PostMapping("/completeFalse")
    @Operation(summary = "驳回或撤回转正流程信息")
    @PreAuthorize("@ss.hasPermission('bpm:oa-entry:complete-false')")
    public CommonResult<Long> completeFalseOaConversion(@Valid @RequestBody BpmTaskApproveReqVO reqVO) {
        return success(oaConversionService.completeFalseOaConversion(getLoginUserId(), reqVO));
    }

    @PostMapping("/updateComplete")
    @Operation(summary = "驳回和撤回后再次提交或关闭转正流程信息")
    @PreAuthorize("@ss.hasPermission('bpm:oa-entry:update-complete')")
    public CommonResult<Long> updateCompleteOaConversion(@Valid @RequestBody OaConversionSaveReqVO createReqVO) {
        return success(oaConversionService.updateCompleteOaConversion(getLoginUserId(), createReqVO, createReqVO.getBpmTaskApproveReqVO()));
    }


    @PostMapping("/create")
    @Operation(summary = "创建转正流程信息")
    @PreAuthorize("@ss.hasPermission('bpm:oa-conversion:create')")
    public CommonResult<Long> createOaConversion(@Valid @RequestBody OaConversionSaveReqVO createReqVO) {
        return success(oaConversionService.createOaConversion(createReqVO));
    }

    @PutMapping("/update")
    @Operation(summary = "更新转正流程信息")
    @PreAuthorize("@ss.hasPermission('bpm:oa-conversion:update')")
    public CommonResult<Boolean> updateOaConversion(@Valid @RequestBody OaConversionSaveReqVO updateReqVO) {
        oaConversionService.updateOaConversion(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @Operation(summary = "删除转正流程信息")
    @Parameter(name = "id", description = "编号", required = true)
    @PreAuthorize("@ss.hasPermission('bpm:oa-conversion:delete')")
    public CommonResult<Boolean> deleteOaConversion(@RequestParam("id") Long id) {
        oaConversionService.deleteOaConversion(id);
        return success(true);
    }

    @GetMapping("/get")
    @Operation(summary = "获得转正流程信息")
    @Parameter(name = "id", description = "编号", required = true, example = "1024")
    @PreAuthorize("@ss.hasPermission('bpm:oa-conversion:query')")
    public CommonResult<OaConversionRespVO> getOaConversion(@RequestParam("id") Long id) {
        OaConversionDO oaConversion = oaConversionService.getOaConversion(id);
        return success(BeanUtils.toBean(oaConversion, OaConversionRespVO.class));
    }

    @GetMapping("/page")
    @Operation(summary = "获得转正流程信息分页")
    @PreAuthorize("@ss.hasPermission('bpm:oa-conversion:query')")
    public CommonResult<PageResult<OaConversionRespVO>> getOaConversionPage(@Valid OaConversionPageReqVO pageReqVO) {
        PageResult<OaConversionDO> pageResult = oaConversionService.getOaConversionPage(pageReqVO);
        return success(BeanUtils.toBean(pageResult, OaConversionRespVO.class));
    }

    @GetMapping("/export-excel")
    @Operation(summary = "导出转正流程信息 Excel")
    @PreAuthorize("@ss.hasPermission('bpm:oa-conversion:export')")
    @ApiAccessLog(operateType = EXPORT)
    public void exportOaConversionExcel(@Valid OaConversionPageReqVO pageReqVO,
              HttpServletResponse response) throws IOException {
        pageReqVO.setPageSize(PageParam.PAGE_SIZE_NONE);
        List<OaConversionDO> list = oaConversionService.getOaConversionPage(pageReqVO).getList();
        // 导出 Excel
        ExcelUtils.write(response, "转正流程信息.xls", "数据", OaConversionRespVO.class,
                        BeanUtils.toBean(list, OaConversionRespVO.class));
    }

}