package cn.iocoder.yudao.module.bpm.service.oa.conversion;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.IdUtil;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.module.bpm.api.task.BpmProcessInstanceApi;
import cn.iocoder.yudao.module.bpm.api.task.dto.BpmProcessInstanceCreateReqDTO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.conversion.vo.OaConversionPageReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.conversion.vo.OaConversionSaveReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.task.vo.task.BpmTaskApproveReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.task.vo.task.BpmTaskReturnReqVO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.conversion.OaConversionDO;
import cn.iocoder.yudao.module.bpm.dal.mysql.oa.conversion.OaConversionMapper;
import cn.iocoder.yudao.module.bpm.service.task.BpmTaskService;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.flowable.engine.TaskService;
import org.flowable.task.api.Task;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.bpm.enums.ErrorCodeConstants.OA_CONVERSION_NOT_EXISTS;

/**
 * 转正流程信息 Service 实现类
 *
 * @author 芋道源码
 */
@Service
@Validated
public class OaConversionServiceImpl extends ServiceImpl<OaConversionMapper, OaConversionDO> implements OaConversionService {

    /**
     * OA 转正对应的流程定义 KEY
     */
    public static final String PROCESS_KEY = "oa_conversion";

    @Resource
    private OaConversionMapper oaConversionMapper;

    @Resource
    private BpmProcessInstanceApi processInstanceApi;

    @Resource
    private BpmTaskService bpmTaskService;

    @Resource
    private TaskService taskService;


    @Override
    @Transactional
    public Long startOaConversion(Long userId, OaConversionSaveReqVO createReqVO) {
        //判断是否已经发起流程
        Boolean auditPass = createReqVO.getAuditPass();
        // 创建单据uuid
        String uuid = IdUtil.fastSimpleUUID();
        OaConversionDO oaConversion = BeanUtils.toBean(createReqVO, OaConversionDO.class);
        if (StringUtils.isBlank(oaConversion.getConversionId())) {
            oaConversion.setConversionId(uuid);
        }
        // 保存或更新表单信息
        this.saveOrUpdate(oaConversion);

        // 提交
        if (auditPass) {
            // 发起 BPM 流程
            Map<String, Object> processInstanceVariables = new HashMap<>();
            processInstanceVariables.put("auditPass", auditPass);
            //添加审批人信息
            Map<String, List<Long>> startUserSelectAssignees = new HashMap<>();
            startUserSelectAssignees.put("approver", createReqVO.getStartUserSelectAssignees());
            String processInstanceId = processInstanceApi
                    .createProcessInstance(userId,
                            new BpmProcessInstanceCreateReqDTO()
                                    .setProcessDefinitionKey(PROCESS_KEY)
                                    .setVariables(processInstanceVariables)
                                    .setBusinessKey(String.valueOf(oaConversion.getId()))
                                    .setStartUserSelectAssignees(startUserSelectAssignees));

            //获取下一个审批人
            Task task = taskService.createTaskQuery().processInstanceId(processInstanceId).singleResult();

            // 将工作流的编号，单据状态 最后一次审批时间 当前审批人  更新到 OA 转正中
            oaConversionMapper.updateById(new OaConversionDO()
                    .setId(oaConversion.getId())
                    .setProcInstId(processInstanceId)
                    .setAuditStatus("1")
                    .setConversionId(uuid)
                    .setFinalAuditDate(LocalDateTime.now())
                    .setCurrentAuditEmployeeId(Long.valueOf(task.getAssignee())));

            // TODO 发送通知

        }else {
            // 暂存，不发起流程
            // 将单据状态 更新到 OA转正中
            oaConversionMapper.updateById(new OaConversionDO()
                    .setId(oaConversion.getId())
                    .setAuditStatus("0"));
        }

        // TODO 保存或更新附件uuid
        if (CollectionUtil.isNotEmpty(createReqVO.getFileIdList())) {
        }

        // 返回
        return oaConversion.getId();
    }
    @Override
    public Long completeTrueOaConversion(Long userId, BpmTaskApproveReqVO reqVO) {
        Task currentTask = bpmTaskService.getTask(reqVO.getId());
        // 审批同意
        bpmTaskService.approveTask(userId, reqVO);

        Task nextTask = taskService.createTaskQuery().processInstanceId(currentTask.getProcessInstanceId()).singleResult();
        if (nextTask != null) {
            LambdaUpdateWrapper<OaConversionDO> updateWrapper = new LambdaUpdateWrapper<>();
            updateWrapper.set(OaConversionDO::getAuditStatus, "2")
                    .set(OaConversionDO::getCurrentAuditEmployeeId, nextTask.getAssignee())
                    .set(OaConversionDO::getFinalAuditDate, new Date())
                    .eq(OaConversionDO::getProcInstId, currentTask.getProcessInstanceId());
            oaConversionMapper.update(updateWrapper);
        } else {
            LambdaUpdateWrapper<OaConversionDO> updateWrapper = new LambdaUpdateWrapper<>();
            updateWrapper.set(OaConversionDO::getAuditStatus, "3")
                    .set(OaConversionDO::getCurrentAuditEmployeeId, null)
                    .set(OaConversionDO::getFinalAuditDate, new Date())
                    .eq(OaConversionDO::getProcInstId, currentTask.getProcessInstanceId());
            oaConversionMapper.update(updateWrapper);

            // TODO 复制业务单据信息到业务模块单据表

        }
        // 发送通知

        // 返回
        return 1L;
    }

    @Override
    public Long completeFalseOaConversion(Long userId, BpmTaskApproveReqVO reqVO) {

        Task currentTask = bpmTaskService.getTask(reqVO.getId());

        BpmTaskReturnReqVO reqVO2 = BeanUtils.toBean(reqVO, BpmTaskReturnReqVO.class);
        reqVO2.setTargetTaskDefinitionKey("modifyApply");
        bpmTaskService.returnTask(userId, reqVO2);

        Task nextTask = taskService.createTaskQuery().processInstanceId(currentTask.getProcessInstanceId()).singleResult();

        // 更新单据状态，当前处理人，最后处理时间
        LambdaUpdateWrapper<OaConversionDO> updateWrapper = new LambdaUpdateWrapper<>();
        updateWrapper.set(OaConversionDO::getCurrentAuditEmployeeId, nextTask.getAssignee())
                .set(OaConversionDO::getFinalAuditDate, new Date())
                .set(OaConversionDO::getAuditStatus, "0")
                .eq(OaConversionDO::getProcInstId, currentTask.getProcessInstanceId());
        oaConversionMapper.update(updateWrapper);

        // 发送通知

        // 返回
        return 1L;
    }

    @Override
    public Long updateCompleteOaConversion(Long userId, OaConversionSaveReqVO createReqVO, BpmTaskApproveReqVO reqVO) {
        // 判断是否已经发起流程
        Boolean auditPass = createReqVO.getAuditPass();

        OaConversionDO oaConversion = BeanUtils.toBean(createReqVO, OaConversionDO.class);

        Task currentTask = taskService.createTaskQuery().processInstanceId(oaConversion.getProcInstId()).singleResult();

        //审批通过
        bpmTaskService.approveTask(userId, reqVO);

        // 提交
        if (auditPass) {
            Task nextTask = taskService.createTaskQuery().processInstanceId(currentTask.getProcessInstanceId()).singleResult();

            oaConversion.setAuditStatus("1");
            oaConversion.setCurrentAuditEmployeeId(Long.valueOf(nextTask.getAssignee()));
            oaConversion.setFinalAuditDate(LocalDateTime.now());
            oaConversionMapper.updateById(oaConversion);

        } else {
            OaConversionDO oaConversionClose = new OaConversionDO();
            oaConversionClose.setId(oaConversion.getId());
            oaConversionClose.setAuditStatus("4");
            oaConversionClose.setCurrentAuditEmployeeId(null);
            oaConversionClose.setFinalAuditDate(LocalDateTime.now());
            oaConversionMapper.updateById(oaConversionClose);
        }

        return oaConversion.getId();
    }



    @Override
    public Long createOaConversion(OaConversionSaveReqVO createReqVO) {
        // 插入
        OaConversionDO oaConversion = BeanUtils.toBean(createReqVO, OaConversionDO.class);
        oaConversionMapper.insert(oaConversion);
        // 返回
        return oaConversion.getId();
    }

    @Override
    public void updateOaConversion(OaConversionSaveReqVO updateReqVO) {
        // 校验存在
        validateOaConversionExists(updateReqVO.getId());
        // 更新
        OaConversionDO updateObj = BeanUtils.toBean(updateReqVO, OaConversionDO.class);
        oaConversionMapper.updateById(updateObj);
    }

    @Override
    public void deleteOaConversion(Long id) {
        // 校验存在
        validateOaConversionExists(id);
        // 删除
        oaConversionMapper.deleteById(id);
    }

    private void validateOaConversionExists(Long id) {
        if (oaConversionMapper.selectById(id) == null) {
            throw exception(OA_CONVERSION_NOT_EXISTS);
        }
    }

    @Override
    public OaConversionDO getOaConversion(Long id) {
        return oaConversionMapper.selectById(id);
    }

    @Override
    public PageResult<OaConversionDO> getOaConversionPage(OaConversionPageReqVO pageReqVO) {
        return oaConversionMapper.selectPage(pageReqVO);
    }

}